"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiscordClient = void 0;
const ws_1 = require("ws");
const events_1 = require("events");
class DiscordClient extends events_1.EventEmitter {
    constructor(discordToken) {
        super();
        this.discordToken = discordToken;
        this.seq = null;
        this.session_id = null;
        this.ack = false;
        this.ws = new ws_1.WebSocket('wss://gateway.discord.gg/?v=6&encoding=json');
    }
    connect() {
        this.ws = new ws_1.WebSocket('wss://gateway.discord.gg/?v=6&encoding=json');
        this.ws.addEventListener('error', event => {
            console.error(`WebSocket error: ${event}`);
        });
        this.ws.on('message', (data) => {
            const payload = JSON.parse(data);
            const { op, d, s, t } = payload;
            this.seq = s ? s : this.seq;
            if (op == 1) {
                this.heartbeat();
            }
            else if (op == 9) {
                setTimeout(() => {
                    this.identify();
                }, 3000);
            }
            else if (op == 10) {
                this.heartbeatTimer = setInterval(() => {
                    this.heartbeat();
                }, d.heartbeat_interval);
                if (this.session_id && this.seq) {
                    this.ws.send(JSON.stringify({
                        'op': 6,
                        'd': {
                            'token': this.discordToken,
                            'session_id': this.session_id,
                            'seq': this.seq
                        }
                    }));
                }
                else {
                    this.identify();
                }
            }
            else if (op == 11) {
                this.ack = true;
            }
            switch (t) {
                case 'READY':
                    this.session_id = d.session_id;
                    break;
                case 'MESSAGE_CREATE':
                    this.emit('messageCreate', d);
                    break;
            }
        });
    }
    heartbeat() {
        this.ws.send(JSON.stringify({
            'op': 1,
            'd': this.seq
        }));
        this.ack = false;
        setTimeout(() => {
            if (!this.ack) {
                this.ws.close();
                this.ack = false;
                if (this.heartbeatTimer) {
                    clearInterval(this.heartbeatTimer);
                }
                this.connect();
            }
        }, 5000);
    }
    identify() {
        this.ws.send(JSON.stringify({
            'op': 2,
            'd': {
                'token': this.discordToken,
                'properties': {
                    '$os': 'linux',
                    '$browser': 'chrome',
                    '$device': 'chrome'
                }
            }
        }));
    }
}
exports.DiscordClient = DiscordClient;
//# sourceMappingURL=DiscordClient.js.map